# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
from six import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String, Multiple


class NaviosBase(Splitted):
    SEPARATOR = "\t"


class NaviosResult(object):
    FIELDS = [
        String("file"),
        String("gate"),
        String("x_param"),
        String("y_param"),
        String("barcode"),
        String("region"),
        Multiple("results", (
            String("resu"),
        ), separator="\t"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class NaviosMultiple(list):

    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: NaviosResult
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str('\t'))
        for line in reader:
            if not len(line):
                continue
            if len(line) in (8, 9):
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {
        "code": "navios",
        "label": "Navios FC50 Beckman"
    }

    def parse(self, data):
        obj = NaviosMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.file.strip() == "File" and m.gate.strip() == "Gate":
                RESULTS_CODES = m.results
                continue
            sample = worklist.get_or_create(
                sample_id=m.barcode.strip(),
                type="patient",
            )
            for i, result in enumerate(m.results):
                if m.region:
                    sample.results.add(
                        code="{}{}".format(m.region.strip(), RESULTS_CODES[i].strip()),
                        value=result.strip(),
                    )
        return worklist
