# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.protocol.mllp import MLLPProtocol


class Atellica800CytoProtocol(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\&|%(sender_id)s|%(receiver_id)s||%(date)s||%(message_type)s|%(date)s|P|2.5\rMSA|AA|%(ack_id)s|\r"""
    HL7_QACK = b"""MSH|^~\&|%(sender_id)s|%(receiver_id)s||%(date)s||%(message_type)s|%(date)s|P|2.5|||NE|AL||ASCII\rMSA|AA\rQAK|LaboratoryOrderQuery|OK\rQPD|WOS^Work Order Step^IHELAW||%(qpd_sid)s"""

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted_payload = payload.splitlines()
        splitted = payload.splitlines()[0].split(b"|")
        qpd_sid = b""
        for line in splitted_payload:
            splitted_qpd = b""
            if b"QPD" in line:
                splitted_qpd = line.split(b"|")
                qpd_sid = splitted_qpd[4]
        date = date or datetime.datetime.now()
        type_message = splitted[8]
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"ack_id": splitted[9],
            b"qpd_sid": qpd_sid,
            b"message_type": self.ACK_MESSAGE_TYPE[self.is_automate],
        }
        if type_message.startswith(b"QBP"):
            context[b"message_type"] = b"RSP^K11^RSP_K11"
            return Atellica800CytoProtocol.HL7_QACK % context
        else:
            context[b"message_type"] = b"ACK^R22^ACK"
            return Atellica800CytoProtocol.HL7_ACK % context
        return self.HL7_ACK % context
