# -*- coding: utf -*-

from __future__ import unicode_literals
import datetime
import time

from driverng.protocol.mllp import MLLPProtocol


class StarletProtocol(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\&|%(sender_id)s|%(sender_label)s|%(receiver_id)s|%(receiver_label)s|%(date)s||ORU^R01|%(control_id)s|P|2.5\rMSA|AA|%(ack_id)s|%(rack_id)s|\r"""

    def __init__(self, **kwargs):
        super(StarletProtocol, self).__init__(**kwargs)

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted = payload.splitlines()[0].split(b"|")
        for line in payload.splitlines():
            if b"OBR" in line.split(b"|")[0]:
                splitted_obr = line.split(b"|")
                if len(splitted_obr) > 3 and len(splitted_obr[3].split(b"^")) == 2:
                    rack_id = splitted_obr[3].split(b"^")[1]
                else:
                    rack_id = b""
                continue
        date = date or datetime.datetime.now()
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"control_id": control_id,
            b"sender_id": splitted[4],
            b"sender_label": splitted[5],
            b"receiver_id": splitted[2],
            b"receiver_label": splitted[3],
            b"ack_id": splitted[9],
            b"rack_id": rack_id,
        }
        return self.HL7_ACK % context

    def recv_trames(self):
        data = self.mllp_readline()
        payload = self.get_payload(data)
        self.write(self.build_trames(self.get_ack_message(payload)))
        # Spécificité non documentée du Starlet, il faut obligatoirement fermer
        # la socket serveur tcp/ip après écriture sur le port
        self.transport.reset()
        return payload

    @staticmethod
    def get_payload(data):
        return data[1:-2]
